@@ -5,7 +5,7 @@ The latest version of this package is available at:
 <https://github.com/jantman/awslimitchecker>
 
 ################################################################################
-Copyright 2015 Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
+Copyright 2015-2018 Jason Antman <jason@jasonantman.com>
 
     This file is part of awslimitchecker, also known as awslimitchecker.
 
@@ -49,13 +49,18 @@ SOURCE_TA = 2
 #: indicates a limit value that came from the service's API
 SOURCE_API = 3
 
+#: indicates a limit value that came from the Service Quotas service
+SOURCE_QUOTAS = 4
+
 
 class AwsLimit(object):
 
     def __init__(self, name, service, default_limit,
                  def_warning_threshold, def_critical_threshold,
                  limit_type=None, limit_subtype=None,
-                 ta_service_name=None, ta_limit_name=None):
+                 ta_service_name=None, ta_limit_name=None,
+                 quotas_service_code=None, quotas_name=None,
+                 quotas_unit='None', quotas_unit_converter=None):
         """
         Describes one specific AWS service limit, as well as its
         current utilization, default limit, thresholds, and any
@@ -63,12 +68,12 @@ class AwsLimit(object):
 
         :param name: the name of this limit (may contain spaces);
           if possible, this should be the name used by AWS, i.e. TrustedAdvisor
-        :type name: string
+        :type name: str
         :param service: the :py:class:`~._AwsService` class that
           this limit is for
         :type service: :py:class:`~._AwsService`
         :param default_limit: the default value of this limit for new accounts
-        :type default_limit: int
+        :type default_limit: :py:obj:`int`, or ``None`` if unlimited
         :param def_warning_threshold: the default warning threshold, as an
           integer percentage.
         :type def_warning_threshold: int
@@ -88,6 +93,28 @@ class AwsLimit(object):
         :param ta_limit_name: The limit name returned by Trusted Advisor for
           this limit, if different from ``name``.
         :type ta_limit_name: str
+        :param quotas_service_code: the Service Quotas service code to
+          retrieve this limit from, if different from the
+          :py:attr:`~._AwsService.quotas_service_code` attribute of
+          ``service``.
+        :type quotas_service_code: str or None
+        :param quotas_name: the Service Quotas quota name to use for this
+          limit, if different from the limit ``name``.
+        :type quotas_name: str or None
+        :param quotas_unit: the Service Quotas quota unit that we need our
+          limit value to be, for quotas that use units. This must be one of
+          the units supported by :py:class:`~.ServiceQuotasClient`. It defaults
+          to the string "None", which (for some strange reason) is what's
+          returned by the Service Quotas API.
+        :type quotas_unit: str
+        :param quotas_unit_converter: A callable to be passed to
+          :py:meth:`~.ServiceQuotasClient.get_quota_value` for unit conversion.
+          Must take three positional arguments: the Service Quotas value for
+          this quota (float), the quota ``Unit`` str, and the return value of
+          :py:meth:`~.quotas_unit`. This callable is responsible for converting
+          the quota value from the quota Unit to this class's expected unit.
+          If they cannot be converted, it should log an error and return None.
+        :type quotas_unit_converter: ``callable``
         :raises: ValueError
         """
         if def_warning_threshold >= def_critical_threshold:
@@ -101,6 +128,7 @@ class AwsLimit(object):
         self.limit_override = None
         self.override_ta = True
         self.ta_limit = None
+        self.ta_unlimited = False
         self.api_limit = None
         self._current_usage = []
         self.def_warning_threshold = def_warning_threshold
@@ -113,6 +141,11 @@ class AwsLimit(object):
         self._criticals = []
         self._ta_service_name = ta_service_name
         self._ta_limit_name = ta_limit_name
+        self._quotas_service_code = quotas_service_code
+        self._quotas_name = quotas_name
+        self._quotas_unit = quotas_unit
+        self.quotas_limit = None
+        self.quotas_unit_converter = quotas_unit_converter
 
     def set_limit_override(self, limit_value, override_ta=True):
         """
@@ -141,6 +174,15 @@ class AwsLimit(object):
         """
         self.ta_limit = limit_value
 
+    def _set_ta_unlimited(self):
+        """
+        Set state to indicate that TrustedAdvisor reports this limit as
+        having no maximum (unlimited).
+
+        This method should only be called by :py:class:`~.TrustedAdvisor`.
+        """
+        self.ta_unlimited = True
+
     def _set_api_limit(self, limit_value):
         """
         Set the value for the limit as reported by the service's API.
@@ -152,30 +194,46 @@ class AwsLimit(object):
         """
         self.api_limit = limit_value
 
+    def _set_quotas_limit(self, limit_value):
+        """
+        Set the value for the limit as reported by the Service Quotas service.
+
+        This method should only be called from the Service class.
+
+        :param limit_value: the Service Quotas limit value
+        :type limit_value: float
+        """
+        self.quotas_limit = limit_value
+
     def get_limit_source(self):
         """
-        Return :py:const:`~awslimitchecker.limit.SOURCE_DEFAULT` if
+        Return :py:data:`~awslimitchecker.limit.SOURCE_DEFAULT` if
         :py:meth:`~.get_limit` returns the default limit,
-        :py:const:`~awslimitchecker.limit.SOURCE_OVERRIDE` if it returns a
+        :py:data:`~awslimitchecker.limit.SOURCE_OVERRIDE` if it returns a
         manually-overridden limit,
-        :py:const:`~awslimitchecker.limit.SOURCE_TA` if it returns a limit from
-        Trusted Advisor, or   :py:const:`~awslimitchecker.limit.SOURCE_API`
-        if it returns a limit retrieved from the service's API.
+        :py:data:`~awslimitchecker.limit.SOURCE_TA` if it returns a limit from
+        Trusted Advisor, :py:data:`~awslimitchecker.limit.SOURCE_API` if it
+        returns a limit retrieved from the service's API, or
+        :py:data:`~.SOURCE_QUOTAS` if it returns a limit from the Service
+        Quotas service.
 
-        :returns: one of :py:const:`~awslimitchecker.limit.SOURCE_DEFAULT`,
-          :py:const:`~awslimitchecker.limit.SOURCE_OVERRIDE`, or
-          :py:const:`~awslimitchecker.limit.SOURCE_TA`, or
-          :py:const:`~awslimitchecker.limit.SOURCE_API`
+        :returns: one of :py:data:`~awslimitchecker.limit.SOURCE_DEFAULT`,
+          :py:data:`~awslimitchecker.limit.SOURCE_OVERRIDE`, or
+          :py:data:`~awslimitchecker.limit.SOURCE_TA`, or
+          :py:data:`~awslimitchecker.limit.SOURCE_API`, or
+          :py:data:`~.awslimitchecker.limit.SOURCE_QUOTAS`
         :rtype: int
         """
         if self.limit_override is not None and (
                 self.override_ta is True or
-                self.ta_limit is None
+                (self.ta_limit is None and self.ta_unlimited is False)
         ):
             return SOURCE_OVERRIDE
         if self.api_limit is not None:
             return SOURCE_API
-        if self.ta_limit is not None:
+        if self.quotas_limit is not None:
+            return SOURCE_QUOTAS
+        if self.ta_limit is not None or self.ta_unlimited is True:
             return SOURCE_TA
         return SOURCE_DEFAULT
 
@@ -183,20 +241,35 @@ class AwsLimit(object):
         """
         Returns the effective limit value for this Limit,
         taking into account limit overrides and Trusted
-        Advisor data.
+        Advisor data. None is returned for limits that are
+        explicitly unlimited.
 
-        :returns: effective limit value
-        :rtype: int
+        :returns: effective limit value, ``int`` or ``None``
         """
         limit_type = self.get_limit_source()
         if limit_type == SOURCE_OVERRIDE:
             return self.limit_override
         elif limit_type == SOURCE_API:
             return self.api_limit
+        elif limit_type == SOURCE_QUOTAS:
+            return self.quotas_limit
         elif limit_type == SOURCE_TA:
+            if self.ta_unlimited is True:
+                return None
             return self.ta_limit
         return self.default_limit
 
+    def has_resource_limits(self):
+        """
+        Determines if this limit contains usages with a specified maximum.
+        Some AWS resources have a limit that is a different for each item.
+
+        :returns: whether of not some resources have a defined maximum
+        :rtype: bool
+        """
+        return any(usage for usage in self._current_usage if
+                   usage.get_maximum())
+
     def get_current_usage(self):
         """
         Get the current usage for this limit, as a list of
@@ -225,7 +298,7 @@ class AwsLimit(object):
         instances in ascending order.
 
         :returns: representation of current usage
-        :rtype: string
+        :rtype: str
         """
         if len(self._current_usage) == 0:
             return '<unknown>'
@@ -238,7 +311,8 @@ class AwsLimit(object):
         )
         return s
 
-    def _add_current_usage(self, value, resource_id=None, aws_type=None):
+    def _add_current_usage(self, value, maximum=None, resource_id=None,
+                           aws_type=None):
         """
         Add a new current usage value for this limit.
 
@@ -254,17 +328,18 @@ class AwsLimit(object):
         :type value: :py:obj:`int` or :py:obj:`float`
         :param resource_id: If there can be multiple usage values for one limit,
           an AWS ID for the resource this instance describes
-        :type resource_id: string
+        :type resource_id: str
         :param aws_type: if ``id`` is not None, the AWS resource type
           that ID represents. As a convention, we use the AWS Resource
           Type names used by
           `CloudFormation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html>`_  # noqa
-        :type aws_type: string
+        :type aws_type: str
         """
         self._current_usage.append(
             AwsLimitUsage(
                 self,
                 value,
+                maximum=maximum,
                 resource_id=resource_id,
                 aws_type=aws_type
             )
@@ -350,10 +425,12 @@ class AwsLimit(object):
         :rtype: bool
         """
         (warn_int, warn_pct, crit_int, crit_pct) = self._get_thresholds()
-        limit = self.get_limit()
         all_ok = True
         for u in self._current_usage:
             usage = u.get_value()
+            limit = u.get_maximum() or self.get_limit()
+            if limit is None:
+                continue
             pct = (usage / (limit * 1.0)) * 100
             if crit_int is not None and usage >= crit_int:
                 self._criticals.append(u)
@@ -417,10 +494,45 @@ class AwsLimit(object):
             return self._ta_limit_name
         return self.name
 
+    @property
+    def quotas_service_code(self):
+        """
+        Return the Service Quotas service code to use for this limit.
+
+        :return: Service Quotas service code
+        :rtype: str
+        """
+        if self._quotas_service_code is not None:
+            return self._quotas_service_code
+        return self.service.quotas_service_code
+
+    @property
+    def quota_name(self):
+        """
+        Return the Service Quotas quota name to use for this limit.
+
+        :return: Service Quotas quota name
+        :rtype: str
+        """
+        if self._quotas_name is not None:
+            return self._quotas_name
+        return self.name
+
+    @property
+    def quotas_unit(self):
+        """
+        Return the Service Quotas unit to use for this limit.
+
+        :return: Service Quotas unit
+        :rtype: str
+        """
+        return self._quotas_unit
+
 
 class AwsLimitUsage(object):
 
-    def __init__(self, limit, value, resource_id=None, aws_type=None):
+    def __init__(self, limit, value, maximum=None, resource_id=None,
+                 aws_type=None):
         """
         This object describes the usage of an AWS resource, with the capability
         of containing information about the resource beyond an integer usage.
@@ -441,17 +553,20 @@ class AwsLimitUsage(object):
         :type limit: :py:class:`~.AwsLimit`
         :param value: the numeric usage value
         :type value: :py:obj:`int` or :py:obj:`float`
+        :param maximum: the numeric maximum value
+        :type maximum: :py:obj:`int` or :py:obj:`float`
         :param resource_id: If there can be multiple usage values for one limit,
           an AWS ID for the resource this instance describes
-        :type resource_id: string
+        :type resource_id: str
         :param aws_type: if ``id`` is not None, the AWS resource type
           that ID represents. As a convention, we use the AWS Resource
           Type names used by
           `CloudFormation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html>`_  # noqa
-        :type aws_type: string
+        :type aws_type: str
         """
         self.limit = limit
         self.value = value
+        self.maximum = maximum
         self.resource_id = resource_id
         self.aws_type = aws_type
 
@@ -464,6 +579,15 @@ class AwsLimitUsage(object):
         """
         return self.value
 
+    def get_maximum(self):
+        """
+        Get the current maximum value
+
+        :returns: current maximum value
+        :rtype: :py:obj:`int` or :py:obj:`float`
+        """
+        return self.maximum
+
     def __str__(self):
         """
         Return a string representation of this object.
@@ -471,7 +595,7 @@ class AwsLimitUsage(object):
         If ``id`` is not set, return ``value`` formatted as a string;
         otherwise, return a string of the format ``id=value``.
 
-        :rtype: string
+        :rtype: str
         """
         s = '{v}'.format(v=self.value)
         if self.resource_id is not None:
